/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QtGui/QLineEdit>
#include <QtNetwork/QHostAddress>

#include <GeoIPCity.h>

#include "debug.h"
#include "kadu.h"
#include "kadu_parser.h"
#include "misc.h"
#include "userinfo.h"
#include "userlistelement.h"

#include "geoip_lookup.h"

static GeoIP *db;

QString geoipLookup(const UserListElement &ule)
{
	kdebugf();

	// answer
	QString lookup;

	// return empty string if user has no IP
	if (!ule.hasIP("Gadu"))
		return lookup;

	// open GeoIPCity.dat file only once
	if ((db == NULL) && ((db = GeoIP_open_type(GEOIP_CITY_EDITION_REV1, GEOIP_INDEX_CACHE|GEOIP_CHECK_CACHE)) == NULL))
		return lookup;

	// search IP in db file
	QByteArray ip = ule.IP("Gadu").toString().toLatin1();
	GeoIPRecord *record = GeoIP_record_by_addr(db, ip.data());
	if (record == NULL)
		return lookup;

	// answer only if fields were not empty
	if ((record->city) && (record->country_code))
		lookup = QString("%1, %2").arg(record->city).arg(record->country_code);

	// cleanup
	GeoIPRecord_delete(record);

	kdebugf2();
	return lookup;
}

GeoIPLookup::GeoIPLookup()
{
	KaduParser::registerTag("geoip", &geoipLookup);

	connect(&UserInfo::createNotifier, SIGNAL(objectCreated(QObject *)), this, SLOT(userInfoCreated(QObject *)));
}

GeoIPLookup::~GeoIPLookup()
{
	KaduParser::unregisterTag("geoip", &geoipLookup);

	disconnect(&UserInfo::createNotifier, SIGNAL(objectCreated(QObject *)), this, SLOT(userInfoCreated(QObject *)));

	// handle cleanup
	if (db != NULL)
		GeoIP_delete(db);
}

void GeoIPLookup::userInfoCreated(QObject *window)
{
	kdebugf();

	UserInfo *info = static_cast<UserInfo *>(window);
	QLineEdit *e_ip = static_cast<QLineEdit *>(info->child("user_ip_address_lineedit"));
	if (!e_ip)
		return;

	QString lookup = geoipLookup(info->user());
	if (!lookup.isEmpty())
		e_ip->setToolTip(lookup);

	kdebugf2();
}
